-- Non-drawing functions/hooks
local SimplerHUD = SimplerHUD
local customhud = customhud
local boss
local bosspatchinfo = {}
SimplerHUD.funcs = {}

local function resetboss(gamemap)
	boss = nil
	bosspatchinfo = {}
end

local function SetCurBoss(mo)
	if (mo.health <= 0) return end
	
	if not ((consoleplayer and consoleplayer.valid)
	and (consoleplayer.mo and consoleplayer.mo.valid)) -- Required or we would have to iterate twice.
		return
	end 
	
	if (P_CheckSight(mo, consoleplayer.mo))
		if ((boss and boss.valid) and not (P_CheckSight(boss, consoleplayer.mo))) // boss exists, but not in line of sight?
		or not (boss and boss.valid) -- or the boss just doesn't exist
			boss = mo
		end
	end
end

local function BossHurt(mo, inf, src)
	if (mo.flags & MF_BOSS)
	and (consoleplayer and consoleplayer.valid)
	and (consoleplayer.mo and consoleplayer.mo.valid)
	and ((inf and inf.valid and inf == consoleplayer.mo)
	or (src and src.valid and src == consoleplayer.mo))
		boss = mo
	end
end

local function PrepareBossPatches(drawer)
	if (#bosspatchinfo == 0)
		table.insert(bosspatchinfo, 1, {patch = drawer.cachePatch("TVEGC0"), bsize = FRACUNIT/4*3})
		table.insert(bosspatchinfo, MT_FANG, {patch = drawer.getSprite2Patch("fang", SPR2_XTRA, 0), bsize = FRACUNIT/2})
		table.insert(bosspatchinfo, MT_METALSONIC_BATTLE, {patch = drawer.getSprite2Patch("metalsonic", SPR2_XTRA, 0), bsize = FRACUNIT/2})
	end
end

addHook("MobjDamage", BossHurt)
addHook("BossThinker", SetCurBoss)

-- Draw related stuff
local clockpatch = "SICLOCK"
local charface = nil
local livesback = nil
local countdowntimer = 0
local stlivex_pat

local function drawRaceNum(v, time) -- Lua implementation of ST_drawRaceNum
	local x
	local y
	local bounce
	local racenum
	local racenumstr = ""

	time = $ + TICRATE
	y = 142
	bounce = TICRATE - (1 + (time % TICRATE))

	if (time/TICRATE == 3)
		racenum = v.cachePatch("RACE3")
	elseif (time/TICRATE == 2)
		racenum = v.cachePatch("RACE2")
	elseif (time/TICRATE == 1)
		racenum = v.cachePatch("RACE1")
	else
		racenum = v.cachePatch("RACEGO")
		racenumstr = "RACEGO"
	end

	if (bounce < 3)
		y = $ - (2 - bounce)
		if (not (paused) and not bounce)
			S_StartSound(nil, ((racenumstr == "RACEGO") and sfx_s3kad or sfx_s3ka7))
		end
	end

	x = ((320 - racenum.width)/2)

	v.draw(x, y, racenum, V_PERPLAYER)
end

local function SimplerHUDScore(v, p)
	if (customhud.CheckType("simplescore") ~= SimplerHUD.modname) return end
	if (customhud.CheckEnabled("simplescore") ~= true) return end
	if (maptol & TOL_NIGHTS) or (G_IsSpecialStage(gamemap))
		return
	end

	if not (SimplerHUD.IsPatchValid(SimplerHUD.patches["SCORE"]))
		SimplerHUD.patches["SCORE"] = v.cachePatch("TVTKC0")
	end
	
	v.drawScaled(24*FRACUNIT, 24*FRACUNIT, FRACUNIT/3*2, SimplerHUD.patches["SCORE"], V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER)
	v.drawNum(104, 12, p.score, V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER)
end

local function SimplerHUDTime(v, p)
	if (customhud.CheckType("simpletime") ~= SimplerHUD.modname) return end
	if (customhud.CheckEnabled("simpletime") ~= true) return end
	if (maptol & TOL_NIGHTS) or (G_IsSpecialStage(gamemap))
		return
	end

	local tics
	local timelimitintics
	local downwards
	local clockcol
	local timerflash
	
	if not (SimplerHUD.IsPatchValid(SimplerHUD.patches["CLOCK"]))
		SimplerHUD.patches["CLOCK"] = v.cachePatch(clockpatch)
	end
	
	if not (SimplerHUD.IsPatchValid(SimplerHUD.patches["STCOL"]))
		SimplerHUD.patches["STCOL"] = v.cachePatch("STTCOLON")
	end

	if (timelimit > 0)
		timelimitintics = timelimit * 60 * TICRATE
	end

	if ((gametyperules & GTR_STARTCOUNTDOWN) and (SimplerHUD.cv_hidetime and SimplerHUD.cv_hidetime.value) and (p.realtime <= (SimplerHUD.cv_hidetime.value*TICRATE)))
		tics = (SimplerHUD.cv_hidetime.value*TICRATE - p.realtime)
			if (tics < 3*TICRATE)
				drawRaceNum(v, tics)
			end
		tics = $ + (TICRATE-1) -- match the race num
		downwards = true
	else
		-- Hidetime finish!
		if ((gametyperules & GTR_STARTCOUNTDOWN) and (p.realtime < (SimplerHUD.cv_hidetime.value+1)*TICRATE))
			drawRaceNum(v, SimplerHUD.cv_hidetime.value*TICRATE - p.realtime)
		end

		-- Time limit?
		if ((gametyperules & GTR_TIMELIMIT) and timelimit)
			if (timelimitintics > p.realtime)
				tics = (timelimitintics - p.realtime)
				if (tics < 3*TICRATE)
					drawRaceNum(v, tics)
				end
				tics = $ + (TICRATE-1) -- match the race num
			else -- Overtime!
				tics = 0
			end
			downwards = true
		elseif (mapheaderinfo[gamemap].countdown)
			tics = (countdowntimer - p.realtime)
		else
			tics = p.realtime
			downwards = false
		end
	end

	timerflash = (downwards and tics < 30*TICRATE and leveltime%10 < 5)
	clockcol = (timerflash) and v.getColormap(TC_DEFAULT, SKINCOLOR_RED) or v.getColormap(TC_DEFAULT, SKINCOLOR_SILVER)

	v.drawScaled(16*FRACUNIT, 36*FRACUNIT, FRACUNIT/3*4, SimplerHUD.patches["CLOCK"], V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER, clockcol)
	
	if (timerflash == false)
		v.drawNum(64, 36, G_TicsToMinutes(tics, true), V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER)
		v.draw(64, 36, SimplerHUD.patches["STCOL"], V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER)
		v.drawPaddedNum(88, 36, G_TicsToSeconds(tics), 2, V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER)
	end
end

local function SimplerHUDRings(v, p)
	if (customhud.CheckType("simplerings") ~= SimplerHUD.modname) return end
	if (customhud.CheckEnabled("simplerings") ~= true) return end
	if (maptol & TOL_NIGHTS) or (G_IsSpecialStage(gamemap))
		return
	end
	
	local s=FRACUNIT/2 --Scale of the ring.
	local ringframe
	
	if not (SimplerHUD.patches["RING"])
		SimplerHUD.patches["RING"] = {}
	end
	
	if not (SimplerHUD.IsPatchValid(SimplerHUD.patches["RING"][0]))
		SimplerHUD.patches["RING"][0] = v.getSpritePatch("TRNG", 0, 1)
	end

	if not (SimplerHUD.IsPatchValid(SimplerHUD.patches["RING"][1]))
		SimplerHUD.patches["RING"][1] = v.getSpritePatch("RING", 0, 1)
	end
	
	ringframe = (p.rings <= 0 and leveltime%10 < 5) and SimplerHUD.patches["RING"][0] or SimplerHUD.patches["RING"][1]
	v.drawScaled(24*FRACUNIT,76*FRACUNIT,s,ringframe,V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER, v.getColormap(TC_DEFAULT, SKINCOLOR_RED))
	v.drawNum(64, 64, p.rings, V_SNAPTOLEFT|V_SNAPTOTOP|V_HUDTRANS|V_PERPLAYER)
end

local function SimplerHUDLives(v, p)
	if (customhud.CheckType("simplelives") ~= SimplerHUD.modname) return end
	if (customhud.CheckEnabled("simplelives") ~= true) return end
	if gametype == GT_ENCORE then return end -- Compatibility with encore mode addon
	if (maptol & TOL_NIGHTS) or (G_IsSpecialStage(gamemap))
		return
	end

	if not (stlivex_pat)
		stlivex_pat = v.cachePatch("STLIVEX")
	end

	local flipface = (SimplerHUD.cvars.faceflip.value) and V_FLIP or 0

	if (p.realmo and p.realmo.valid) then
		livesback = v.cachePatch("STLIVEBK")
		local charfaceflags = V_SNAPTOBOTTOM|V_PERPLAYER|((p.spectator) and V_HUDTRANSHALF or V_HUDTRANS)|flipface
		local getsuperface = false
		local plivesinf = (p.lives == INFLIVES or SimplerHUD.cv_cooplives.string == "Infinite") and true or false
		local lifex
		
		if (flipface) // For some reason V_FLIP moves the patch to the left
			lifex = (hudinfo[HUD_LIVES].x+16)
		else
			lifex = (hudinfo[HUD_LIVES].x)
		end

		if (P_IsValidSprite2(p.realmo, SPR2_XTRA)) then
			getsuperface = (p.powers[pw_super]) and true or false -- Should we get the super face?
			charface = v.getSprite2Patch(p.realmo.skin, SPR2_XTRA, getsuperface, 0)
		else
			charface = v.cachePatch("MISSING")
		end

		v.drawScaled(lifex*FRACUNIT, hudinfo[HUD_LIVES].y*FRACUNIT, FRACUNIT/2, charface, charfaceflags, v.getColormap(p.realmo.skin, (p.spectator) and SKINCOLOR_CLOUDY or p.realmo.color))

		if (G_GametypeUsesLives())
			v.draw(hudinfo[HUD_LIVES].x+22, hudinfo[HUD_LIVES].y+10, v.cachePatch("STLIVEX"), hudinfo[HUD_LIVES].f|V_HUDTRANS|V_PERPLAYER)
			v.drawString(hudinfo[HUD_LIVES].x+46, hudinfo[HUD_LIVES].y+8, (plivesinf) and "\x16" or p.lives, hudinfo[HUD_LIVES].f|V_PERPLAYER|V_HUDTRANS, "right")
		end
	end
end

local function SimplerHUDBoss(v, p)
	if (customhud.CheckType("simplebosscounter") != SimplerHUD.modname) return end
	if (customhud.CheckEnabled("simplebosscounter") != true) return end
			
	if not (boss and boss.valid) return end
			
	if (p == consoleplayer)
		local posy1 = 25*FRACUNIT
		local posy2 = 21
		local posy3 = 19
		local posy4 = 11
		local bsize
		local bosspatch 
		local bosscolor
		
		if (bosspatchinfo[boss.type])
			bosspatch = bosspatchinfo[boss.type].patch
			bsize = bosspatchinfo[boss.type].bsize
			posy1 = $ - 10*FRACUNIT
		else
			bosspatch = bosspatchinfo[1].patch
			bsize = bosspatchinfo[1].bsize
		end	
		
		if (boss.type == MT_METALSONIC_BATTLE)
			bosscolor = v.getColormap(TC_DEFAULT, SKINCOLOR_COBALT)
		end
		
		if (boss.type == MT_FANG)
			bosscolor = v.getColormap(TC_DEFAULT, SKINCOLOR_LAVENDER)
		end
		
		v.drawScaled(293*FRACUNIT, posy1, bsize, bosspatch, V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, bosscolor)
		v.draw(277, posy2, stlivex_pat, V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS) -- woo patch reuse
		v.drawString(261, posy3, boss.health, V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
	end
end

SimplerHUD.funcs[1] = SimplerHUDScore
SimplerHUD.funcs[2] = SimplerHUDTime
SimplerHUD.funcs[3] = SimplerHUDRings
SimplerHUD.funcs[4] = SimplerHUDLives
SimplerHUD.funcs[5] = PrepareBossPatches
SimplerHUD.funcs[6] = SimplerHUDBoss